// Written by Andrey Chicherov, uart@os2.ru

#include <afxwin.h>

#include "ptr.h"
#include "TCRFile.h"
#include "TextViewNG.h"

#include <string.h>


// ----------------------------------------------------------------------------------------------------------------

CString TCRFile::CompressionInfo() {

	CString ret; ret.Format(_T("TCR, packed (%.2f)"),
      (double)m_fp->GetLength()/(double)m_length
	);

    return ret;

}

// ----------------------------------------------------------------------------------------------------------------

bool TCRFile::CheckTCR (CFile& fp) {

	char sign [9]; fp.Seek (0, CFile::begin);
	if (fp.Read (sign,9) != 9) return false;
	
	if (memcmp (sign,"!!8-Bit!!",9))
		return false;

	return true;

}

// ----------------------------------------------------------------------------------------------------------------

bool TCRFile::IsTCR (CFile& fp) {

	bool ret = CheckTCR (fp);
	fp.Seek (0,CFile::begin);
	return ret;

}

// ----------------------------------------------------------------------------------------------------------------

TCRFile::TCRFile (auto_ptr<CFile> fp) : RFile(fp), m_length(0), m_ptr(0) {

	if (CheckTCR (*m_fp)) {

		DWORD offset = 9; unsigned char ch; for (int i = 0; i < 256; i++) {

			if (m_fp->Read (&ch, 1) != 1) goto fail; Dictonary [i] = Buffer<unsigned char>(ch + 1);
			Dictonary [i][0] = ch; if (m_fp->Read (Dictonary [i] + 1, ch) != ch) goto fail;
			offset += ch + 1;

		}

		int nr = (int)((m_fp->GetLength() - offset + 4095) / 4096); m_blocks = Buffer<Block>(nr);
		Buffer<unsigned char> tmp (4096); for (i = 0; i < nr; i++) {

			m_blocks [i].off = offset; m_blocks [i].size = m_fp->Read (tmp, 4096);
			m_blocks [i].uoff = m_length; m_blocks [i].usize = 0;
			for (DWORD j = 0; j < m_blocks [i].size; j++)
				m_blocks [i].usize += Dictonary [tmp[j]] [0];

			m_length += m_blocks [i].usize;
			offset += m_blocks [i].size;

		}

		return;

	}

	fail:

	CTVApp::Barf(_T("Invalid or unsupported TCR file"));

}

// ----------------------------------------------------------------------------------------------------------------

void TCRFile::seek (DWORD pos) {

  if (pos>=m_length)
    m_ptr=m_length;
  else
    m_ptr=pos&BMASK;

}

// ----------------------------------------------------------------------------------------------------------------

DWORD TCRFile::read (void *buf) {

	unsigned char *out = (unsigned char*) buf; DWORD n = BSZ; for (int i = 0; i < m_blocks.size() && n; i++)
	if (m_ptr >= m_blocks[i].uoff && m_ptr < m_blocks[i].uoff + m_blocks[i].usize) {

		Buffer<unsigned char> src (m_blocks[i].size); m_fp->Seek (m_blocks[i].off, CFile::begin);
		if (m_fp->Read (src, m_blocks[i].size) != m_blocks[i].size) return 0;

		Buffer<unsigned char> dst (m_blocks[i].usize); unsigned char *ptr = dst;
		for (DWORD j = 0; j < m_blocks[i].size; j++) {

			unsigned char len = Dictonary [src[j]][0];
			memcpy (ptr, Dictonary [src[j]] + 1, len);
			ptr += len;

		}

		DWORD len = m_blocks[i].usize - (m_ptr - m_blocks[i].uoff); if (len > n) len = n;
		memcpy (out, dst + m_ptr - m_blocks[i].uoff, len); out += len;
		m_ptr += len; n -= len;

	}

	return BSZ - n;

}
